@@ -78,6 +78,11 @@
 #include "ifaddr/ifaddrs.h"
 #endif
 
+#ifdef RFC2553
+#define ADDR_FAMILY_MEMBER ss_family
+#else
+#define ADDR_FAMILY_MEMBER sa_family
+#endif
 
 #if defined(__linux__) && defined(NEED_TIMEVAL)
 struct timeval 
@@ -249,7 +254,6 @@ rfbProcessNewConnection(rfbScreenInfoPtr rfbScreen, int insock)
 {
     const int one = 1;
     int sock = -1;
-    int i;
     
     if((sock = accept(insock, NULL, NULL)) < 0) {
       rfbLogPerror("rfbCheckFds: accept");
@@ -579,6 +583,7 @@ ListenOnTCPPort(rfbScreenInfoPtr rfbScreen, int port, const char *netIface)
   if(netIface == NULL || strlen(netIface) == 0)
   {
 #ifdef ENABLE_IPV6
+    int sock6 = -1;
     struct sockaddr_in6 s6;
 
     memset(&s6, 0, sizeof(s6));
@@ -586,10 +591,10 @@ ListenOnTCPPort(rfbScreenInfoPtr rfbScreen, int port, const char *netIface)
     s6.sin6_port   = htons(port);
     s6.sin6_addr   = in6addr_any;
 
-    sock = NewSocketListenTCP ((struct sockaddr*)&s6, sizeof(s6));
-    rfbLog("Listening IPv{4,6}://*:%d\n", port);
+    sock6 = NewSocketListenTCP ((struct sockaddr*)&s6, sizeof(s6));
+    rfbLog("Listening IPv6://[::]:%d\n", port);
 #endif
-    if(sock < 0) {
+
       struct sockaddr_in s4;
 
       memset(&s4, 0, sizeof(s4));
@@ -599,15 +604,23 @@ ListenOnTCPPort(rfbScreenInfoPtr rfbScreen, int port, const char *netIface)
 
       sock = NewSocketListenTCP ((struct sockaddr*)&s4, sizeof(s4));
       rfbLog("Listening IPv4://0.0.0.0:%d\n", port);
-    }
 
+#ifdef ENABLE_IPV6
+    if(sock6 > 0) {
+       psock[*ptot] = sock6;
+      *ptot        += 1;
+    }
+#endif
     if(sock > 0) {
-      psock[0] = sock;
-     *ptot     = 1;
-      return TRUE;
+       psock[*ptot] = sock;
+      *ptot        += 1;
     }
 
-    rfbLog("Problems in NewSocketListenTCP(), sock=%d\n", sock);
+    if (*ptot)
+      return TRUE;
+
+    /* no need to log sock/sock6, both are -1 here */
+    rfbLog("Problems in NewSocketListenTCP()\n");
     return FALSE;
   }
 
@@ -622,7 +635,7 @@ ListenOnTCPPort(rfbScreenInfoPtr rfbScreen, int port, const char *netIface)
     if (ifa->ifa_addr == NULL || (ifa->ifa_flags & IFF_UP) == 0) 
       continue;
 
-    if (ifa->ifa_addr->sa_family == AF_INET) {
+    if (ifa->ifa_addr->ADDR_FAMILY_MEMBER == AF_INET) {
       struct sockaddr_in *s4 = (struct sockaddr_in*)ifa->ifa_addr;
       s4->sin_port           = htons(port);
 
@@ -636,11 +649,11 @@ ListenOnTCPPort(rfbScreenInfoPtr rfbScreen, int port, const char *netIface)
       }
     }
 #ifdef ENABLE_IPV6            
-    if (ifa->ifa_addr->sa_family == AF_INET6) {
+    if (ifa->ifa_addr->ADDR_FAMILY_MEMBER == AF_INET6) {
       struct sockaddr_in6 *s6 = (struct sockaddr_in6*)ifa->ifa_addr;
       s6->sin6_port           = htons(port);
 
-      if (inet_ntop(ifa->ifa_addr->sa_family, (struct sockaddr*)&s6->sin6_addr, buf, sizeof(buf)) == NULL) {
+      if (inet_ntop(ifa->ifa_addr->ADDR_FAMILY_MEMBER, (struct sockaddr*)&s6->sin6_addr, buf, sizeof(buf)) == NULL) {
         rfbLog("%s: inet_ntop failed!\n", ifa->ifa_name);
         continue; 
       }
@@ -677,6 +690,15 @@ NewSocketListenTCP(struct sockaddr *addr, socklen_t len)
       return -1;
     }
 
+#ifdef ENABLE_IPV6
+    if (addr->sa_family == AF_INET6) {
+#if defined(IPPROTO_IPV6) && defined(IPV6_V6ONLY)
+      setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&one, sizeof(one));
+      /* we cannot really check for errors here */
+#endif
+    }
+#endif
+
     if (bind(sock, addr, len) < 0) {
       close(sock);
       return -1;
